@@ -23,5 +23,12 @@ def more_interesting_app(environ, start_response):
     return [pformat(environ).encode('utf-8')]
 
 
+def post_status_headers_app(environ, start_response):
+    headers = []
+    start_response('200 OK', headers)
+    headers.append(('Content-type', 'text/plain'))
+    return [b'WSGI intercept successful!\n']
+
+
 def raises_app(environ, start_response):
     raise TypeError("bah")
